<?php

/**
 * Copyright 2017 MICRORISC s.r.o.
 * Copyright 2017-2019 IQRF Tech s.r.o.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
declare(strict_types = 1);

namespace App\ConfigModule\Models;

use App\ConfigModule\Exceptions\InvalidTaskMessageException;
use App\CoreModule\Exceptions\InvalidJsonException;
use App\CoreModule\Exceptions\NonexistentJsonSchemaException;
use App\CoreModule\Models\CommandManager;
use App\CoreModule\Models\JsonFileManager;
use App\ServiceModule\Exceptions\UnsupportedInitSystemException;
use App\ServiceModule\Models\ServiceManager;
use Nette\IOException;
use Nette\Utils\Finder;
use Nette\Utils\JsonException;
use Nette\Utils\Strings;
use SplFileInfo;
use stdClass;

/**
 * Scheduler configuration manager
 */
class SchedulerManager {

	/**
	 * @var GenericManager Generic config manager
	 */
	private $genericConfigManager;

	/**
	 * @var JsonFileManager JSON file manager
	 */
	private $fileManager;

	/**
	 * @var string File name (without .json)
	 */
	private $fileName;

	/**
	 * @var SchedulerSchemaManager Scheduler JSON schema manager
	 */
	private $schemaManager;

	/**
	 * @var ServiceManager IQRF Gateway Daemon's service manager
	 */
	private $serviceManager;

	/**
	 * @var TaskTimeManager Scheduler's task time specification manager
	 */
	private $timeManager;

	/**
	 * Constructor
	 * @param MainManager $mainManager Main configuration manager
	 * @param GenericManager $genericManager Generic configuration manager
	 * @param TaskTimeManager $timeManager Scheduler's task time specification manager
	 * @param ServiceManager $serviceManager IQRF Gateway Daemon's service manager
	 * @param CommandManager $commandManager Command manager
	 * @param SchedulerSchemaManager $schemaManager Scheduler JSON schema manager
	 */
	public function __construct(MainManager $mainManager, GenericManager $genericManager, TaskTimeManager $timeManager, ServiceManager $serviceManager, CommandManager $commandManager, SchedulerSchemaManager $schemaManager) {
		$this->genericConfigManager = $genericManager;
		$this->serviceManager = $serviceManager;
		$this->timeManager = $timeManager;
		$cacheDir = $mainManager->getCacheDir();
		if (!is_readable($cacheDir) || !is_writable($cacheDir)) {
			$commandManager->run('chmod 777 ' . $cacheDir, true);
		}
		$path = $cacheDir . '/scheduler/';
		$this->fileManager = new JsonFileManager($path, $commandManager);
		$this->schemaManager = $schemaManager;
	}

	/**
	 * Deletes a task
	 * @param int $id Task ID
	 */
	public function delete(int $id): void {
		$files = $this->getTaskFiles();
		if (isset($files[$id])) {
			$this->fileManager->delete($files[$id]);
			try {
				$this->serviceManager->restart();
			} catch (UnsupportedInitSystemException $e) {
				// Do nothing
			}
		}
	}

	/**
	 * Gets task's files
	 * @return array<string> Files with tasks
	 */
	public function getTaskFiles(): array {
		$dir = $this->fileManager->getDirectory();
		$files = [];
		/**
		 * @var SplFileInfo $file File info
		 */
		foreach (Finder::findFiles('*.json')->from($dir) as $file) {
			$dirPathPattern = ['~^' . realpath($dir) . '/~', '/.json$/'];
			$fileName = Strings::replace($file->getRealPath(), $dirPathPattern, '');
			try {
				$json = $this->fileManager->read($fileName);
				if (array_key_exists('taskId', $json)) {
					$files[$json['taskId']] = $fileName;
				}
			} catch (IOException | JsonException $e) {
				// Skip invalid JSON files
			}
		}
		asort($files);
		return $files;
	}

	/**
	 * Gets available messagings
	 * @return array<array<string>> Available messagings
	 * @throws JsonException
	 */
	public function getMessagings(): array {
		$messagings = $this->genericConfigManager->getMessagings();
		unset($messagings['config.udp.title']);
		return $messagings;
	}

	/**
	 * Gets scheduler's services
	 * @return array<string> Scheduler's services
	 * @throws JsonException
	 */
	public function getServices(): array {
		$services = [];
		$this->genericConfigManager->setComponent('iqrf::SchedulerMessaging');
		foreach ($this->genericConfigManager->list() as $instance) {
			$services[] = $instance['instance'];
		}
		return $services;
	}

	/**
	 * Gets tasks
	 * @return array<mixed> Tasks
	 */
	public function list(): array {
		$tasks = [];
		foreach (array_keys($this->getTaskFiles()) as $id) {
			try {
				$data = $this->load($id);
				$task = [
					'id' => $data->taskId,
					'time' => $this->timeManager->getTime($data),
					'service' => $data->clientId,
					'messaging' => $this->getTaskMessagings($data->task),
					'mType' => $this->getTaskMessageTypes($data->task),
				];
				$tasks[] = $task;
			} catch (InvalidJsonException | IOException | JsonException $e) {
				// Do nothing
			}
		}
		return $tasks;
	}

	/**
	 * Returns messagings used in tasks
	 * @param array<mixed> $tasks Tasks
	 * @return string Messagings used in tasks
	 */
	private function getTaskMessagings(array $tasks): string {
		$messagings = [];
		foreach ($tasks as $task) {
			$messagings[] = $task->messaging;
		}
		return implode(', ', $messagings);
	}

	/**
	 * Returns message types used in tasks
	 * @param array<mixed> $tasks Tasks
	 * @return string Message types used in tasks
	 */
	private function getTaskMessageTypes(array $tasks): string {
		$mTypes  = [];
		foreach ($tasks as $task) {
			$mTypes[] = $task->message->mType;
		}
		return implode(', ', $mTypes);
	}

	/**
	 * Loads the task's configuration
	 * @param int $id Task ID
	 * @return stdClass Array for the form
	 * @throws InvalidJsonException
	 * @throws InvalidTaskMessageException
	 * @throws JsonException
	 * @throws NonexistentJsonSchemaException
	 */
	public function load(int $id): stdClass {
		$files = $this->getTaskFiles();
		if (!isset($files[$id])) {
			return new stdClass();
		}
		$this->fileName = strval($files[$id]);
		$config = $this->fileManager->read($this->fileName, false);
		$this->schemaManager->validate($config);
		$this->timeManager->cronToString($config);
		$this->fixTasks($config);
		return $config;
	}

	/**
	 * Fixes the HWPID format
	 * @param int|null $hwpId HWPID to fix
	 * @return string Fixed HWPID
	 */
	public function fixHwpid(?int $hwpId = null): string {
		$data = Strings::padLeft(dechex($hwpId & 255), 2, '0') . '.';
		return $data . Strings::padLeft(dechex($hwpId >> 8), 2, '0');
	}

	/**
	 * Fixes the task specification
	 * @param stdClass $config Scheduler task
	 */
	public function fixTasks(stdClass &$config): void {
		if (!is_array($config->task) && isset($config->task->message)) {
			$config->task = [$config->task];
		}
	}

	/**
	 * Saves the task's configuration
	 * @param stdClass $config Task's configuration
	 * @throws InvalidTaskMessageException
	 * @throws JsonException
	 */
	public function save(stdClass $config): void {
		if (!isset($this->fileName)) {
			$this->fileName = strval($config->taskId);
		}
		foreach ($config->task as &$task) {
			if (!isset($task->message->data->timeout)) {
				unset($task->message->data->timeout);
			}
		}
		$this->timeManager->cronToArray($config);
		if (!isset($config->timeSpec->period)) {
			$config->timeSpec->period = 0;
		}
		$this->schemaManager->validate($config);
		$this->fileManager->write($this->fileName, $config);
	}

}
