<?php

/**
 * Copyright 2017 MICRORISC s.r.o.
 * Copyright 2017-2019 IQRF Tech s.r.o.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
declare(strict_types = 1);

namespace App\ConfigModule\Models;

use DateTime;
use Nette\Utils\Arrays;
use Nette\Utils\JsonException;

/**
 * WebSocket configuration manager
 */
class WebSocketManager {

	/**
	 * @var array<string> WebSocket components
	 */
	private $components = [
		'messaging' => 'iqrf::WebsocketMessaging',
		'service' => 'shape::WebsocketCppService',
	];

	/**
	 * @var array<string> WebSocket messaging and service file names
	 */
	private $fileNames;

	/**
	 * @var GenericManager Generic configuration manager
	 */
	private $genericManager;

	/**
	 * @var array<string,string|null> WebSocket instances
	 */
	private $instances = [];

	/**
	 * @var ComponentSchemaManager JSON schema manager
	 */
	private $schemaManager;

	/**
	 * Constructor
	 * @param GenericManager $genericManager Generic configuration manager
	 * @param ComponentSchemaManager $schemaManager JSON schema manager
	 */
	public function __construct(GenericManager $genericManager, ComponentSchemaManager $schemaManager) {
		$this->genericManager = $genericManager;
		$this->schemaManager = $schemaManager;
	}

	/**
	 * Deletes a configuration
	 * @param int $id WebSocket interface ID
	 * @throws JsonException
	 */
	public function delete(int $id): void {
		$this->genericManager->setComponent($this->components['messaging']);
		$instances = $this->genericManager->getInstanceFiles();
		$this->fileNames['messaging'] = Arrays::pick($instances, $id);
		$messaging = $this->genericManager->read($this->fileNames['messaging']);
		$instance = $messaging['RequiredInterfaces'][0]['target']['instance'];
		$this->genericManager->deleteFile($this->fileNames['messaging']);
		$this->genericManager->deleteFile($this->getServiceFile($instance));
	}

	/**
	 * Gets WebSocket service file name by instance name
	 * @param string $instanceName Instance name
	 * @return string|null WebSocket service file name
	 * @throws JsonException
	 */
	public function getServiceFile(string $instanceName): ?string {
		$this->genericManager->setComponent($this->components['service']);
		$services = $this->genericManager->getInstanceFiles();
		foreach ($services as $service) {
			$json = $this->genericManager->read($service);
			if (Arrays::pick($json, 'instance') === $instanceName) {
				return $service;
			}
		}
		return null;
	}

	/**
	 * Saves the configuration
	 * @param array<mixed> $array Websocket settings
	 * @throws JsonException
	 */
	public function save(array $array): void {
		$timestamp = (new DateTime())->getTimestamp();
		if (!isset($this->instances['messaging'])) {
			$this->instances['messaging'] = 'WebsocketMessaging' . $timestamp;
		}
		if (!isset($this->instances['service'])) {
			$this->instances['service'] = 'WebsocketCppService' . $timestamp;
		}
		$settings = [
			'messaging' => $this->createMessaging($array),
			'service' => $this->createService($array),
		];
		foreach ($settings as $component => $config) {
			$this->schemaManager->setSchema($this->components[$component]);
			$this->schemaManager->validate((object) $config);
		}
		$messagingFileName = $this->fileNames['messaging'] ?? 'iqrf__' . $this->instances['messaging'];
		$serviceFileName = $this->fileNames['service'] ?? 'shape__' . $this->instances['service'];
		$this->genericManager->setComponent($this->components['service']);
		$this->genericManager->save($settings['service'], $serviceFileName);
		$this->genericManager->setComponent($this->components['messaging']);
		$this->genericManager->save($settings['messaging'], $messagingFileName);
	}

	/**
	 * Creates a messaging configuration
	 * @param array<mixed> $values Values from form
	 * @return array<mixed> Messaging configuration
	 */
	private function createMessaging(array $values): array {
		return [
			'component' => $this->components['messaging'],
			'instance' => $this->instances['messaging'],
			'acceptAsyncMsg' => $values['acceptAsyncMsg'],
			'RequiredInterfaces' => [
				(object) [
					'name' => 'shape::IWebsocketService',
					'target' => (object) [
						'instance' => $this->instances['service'],
					],
				],
			],
		];
	}

	/**
	 * Creates a service configuration
	 * @param array<mixed> $values Values from the form
	 * @return array<mixed> Service configuration
	 */
	private function createService(array $values): array {
		return [
			'component' => $this->components['service'],
			'instance' => $this->instances['service'],
			'WebsocketPort' => $values['port'],
			'acceptOnlyLocalhost' => $values['acceptOnlyLocalhost'],
		];
	}

	/**
	 * Gets WebSocket instances
	 * @return array<mixed> WebSocket instances
	 * @throws JsonException
	 */
	public function list(): array {
		$this->genericManager->setComponent($this->components['messaging']);
		$files = $this->genericManager->getInstanceFiles();
		$instances = [];
		foreach (array_keys($files) as $id) {
			$instances[] = Arrays::mergeTree(['id' => $id], $this->load($id));
		}
		return $instances;
	}

	/**
	 * Loads a configuration
	 * @param int $id WebSocket interface ID
	 * @return array<mixed> Array for form
	 * @throws JsonException
	 */
	public function load(int $id): array {
		$this->genericManager->setComponent($this->components['messaging']);
		$instances = $this->genericManager->getInstanceFiles();
		$this->fileNames['messaging'] = Arrays::pick($instances, $id);
		$messaging = $this->genericManager->read($this->fileNames['messaging']);
		$serviceInstance = $messaging['RequiredInterfaces'][0]['target']['instance'];
		$this->fileNames['service'] = $this->getServiceFile($serviceInstance);
		$this->genericManager->setComponent($this->components['service']);
		$service = $this->genericManager->read($this->fileNames['service']);
		$this->instances = [
			'messaging' => $messaging['instance'],
			'service' => $service['instance'],
		];
		return [
			'messagingInstance' => $messaging['instance'],
			'acceptAsyncMsg' => $messaging['acceptAsyncMsg'],
			'serviceInstance' => $service['instance'],
			'port' => $service['WebsocketPort'],
			'acceptOnlyLocalhost' => $service['acceptOnlyLocalhost'] ?? false,
		];
	}

}
